;8.3. Programa de control de un microcontrolador en un entorno multiprocesa-
;dor.
;Modo multiproceso de la UART en un 8051.
;En este caso, el microcontrolador esclavo tiene asignado una direccin cuyo
;   cdigo es 07.
;Unicamente responder a la llamada del PC maestro cuando ste solicite su
;   atencin.
;Para ello el maestro deber enviar por el bus serie el valor de su cdigo
;   asociado.
;El microcontrolador esclavo borrar su bit SM2, y estar en disposicin de 
;   aceptar todos los datos que le envie el maestro. Los datos tendrn el 
;   bit TB8 puesto a 0. De esta forma, el resto de los microcontroladores
;   harn caso omiso a la comunicacin.

          .TITLE    MULTIPRO.ASM
          .ORG      0000H          ;vector reset
          JMP       0200H
          .ORG      0023H          ;vector de la interrupcin serie
          JMP       INTSERIE
;-----------------------
; Definicin de valores
;-----------------------
CR:       .EQU      13             ;retorno de carro
LF:       .EQU      10             ;avance de lnea
PCON:     .EQU      87H            ;PCON no est predefinido
ORDEN:    .DB       CR,LF,"Tarjeta n7 activada",CR,LF,"Orden:",00
ERROR:    .DB       CR,LF,"Orden desconocida",CR,LF
          .DB       "Las rdenes conocidas son: A=> Activar P2.2",CR,LF
          .DB       "                           D=> Desactivar P2.2",CR,LF,00      
CONTROL:  .REG      P2.0           ;control del transmisor SN75176
;--------------------
; Programa principal
;--------------------
          .ORG      0200H
          MOV       SCON,#0F0H     ;inicializa el SCON: REN=1, modo 3
          MOV       TMOD,#20H      ;timer 1 en modo 2 "autorrecarga"
          MOV       TH1,#0FDH      ;velocidad de 9.600 baudios
          ANL       PCON,#7FH      ;SMOD=0
          SETB      TR1            ;activa el T1
          MOV       IE,#90H        ;habilita la interrupcin serie
INICIO:   CPL       P2.1           ;hace parpadear el P2.1
          CALL      RETARDO
          JMP       INICIO
;----------------------
; Subrutina de retardo
;----------------------
RETARDO:  MOV       R0,#02         ;temporiza aproximadamente 4 segundos
SALTO1$:  MOV       R1,#0FFH
SALTO2$:  MOV       R2,#0FFH
          DJNZ      R2,$
          DJNZ      R1,SALTO2$
          DJNZ      R0,SALTO1$
          RET
;--------------------
; Interrupcin serie
;--------------------
;Slamente se activar si se recibe un byte con el 9 bit =1
;Se comprobar que el cdigo de direccin recibido es el de la terjeta.
;En caso contrario saldr de la interrupcin.
INTSERIE: PUSH      A              ;salvaramos los registros necesarios
          PUSH      PSW
          MOV       A,SBUF         ;obtenemos el cdigo de direccin y si el
          CJNE      A,#07H,FININT  ;mensaje no es el suyo, sale de la inter.
          CLR       SM2            ;si lo es, borra SM2
          SETB      CONTROL        ;habilitamos el transmisor
PET_ORD:  MOV       DPTR,#ORDEN    ;y pedimos que nos envien la orden
          CALL      SAL_SER        ;sacamos el mensaje
          JNB       RI,$           ;espera recepcin de la orden
          MOV       A,SBUF         ;orden recibida
          CLR       RI             ;borro RI para prxima recepcin
          CJNE      A,#"A",SALTO1$
          SETB      P2.2           ;activa el P2.2
          JMP       SALIR
SALTO1$:  CJNE      A,"D",SALTO2$
          CLR       P2.2           ;desactiva el P2.2
          JMP       SALIR
SALTO2$:  MOV       DPTR,#ERROR    ;avisamos de que la orden era incorrecta
          CALL      SAL_SER        ;y volvemos a pedir que nos envien otra
          JMP       PET_ORD
SALIR:    CLR       CONTROL
          SETB      SM2            ;activamos SM2
FININT:   POP       PSW            ;recuperaramos los registros guardados
          POP       A
          CLR       RI             ;borramos RI para poder volver a recibir
          RETI
;-------------------------------
; Subrutina para sacar mensajes
;-------------------------------
SAL_SER:  MOV       A,#00
          MOVC      A,@A+DPTR
          JZ        FINMEN
          MOV       SBUF,A         ;transmite el caracter del acumulador
          JNB       TI,$           ;espera que termine de transmitirlo
          CLR       TI             ;borra TI para poder transmitir de nuevo
          INC       DPTR           ;apuntamos al siguiente caracter
          JMP       SAL_SER
FINMEN:   RET
          .END
